@@ -127,13 +127,14 @@ static int curl_progress(void *file, double dltotal, double dlnow,
 	return 0;
 }
 
-static int curl_gethost(const char *url, char *buffer)
+static int curl_gethost(const char *url, char *buffer, size_t buf_len)
 {
 	size_t hostlen;
 	char *p, *q;
 
 	if(strncmp(url, "file://", 7) == 0) {
-		strcpy(buffer, _("disk"));
+		p = _("disk");
+		hostlen = strlen(p);
 	} else {
 		p = strstr(url, "//");
 		if(!p) {
@@ -154,14 +155,15 @@ static int curl_gethost(const char *url, char *buffer)
 			hostlen -= q - p + 1;
 			p = q + 1;
 		}
-
-		if(hostlen > 255) {
-			/* buffer overflow imminent */
-			return 1;
-		}
-		snprintf(buffer, hostlen + 1, "%s", p);
 	}
 
+	if(hostlen > buf_len - 1) {
+		/* buffer overflow imminent */
+		return 1;
+	}
+	memcpy(buffer, p, hostlen);
+	buffer[hostlen] = '\0';
+
 	return 0;
 }
 
@@ -176,6 +178,14 @@ static int utimes_long(const char *path, long seconds)
 	return 0;
 }
 
+/* prefix to avoid possible future clash with getumask(3) */
+static mode_t _getumask(void)
+{
+	mode_t mask = umask(0);
+	umask(mask);
+	return mask;
+}
+
 static size_t parse_headers(void *ptr, size_t size, size_t nmemb, void *user)
 {
 	size_t realsize = size * nmemb;
@@ -293,9 +303,12 @@ static FILE *create_tempfile(struct dload_payload *payload, const char *localpat
 	MALLOC(randpath, len, RET_ERR(payload->handle, ALPM_ERR_MEMORY, NULL));
 	snprintf(randpath, len, "%salpmtmp.XXXXXX", localpath);
 	if((fd = mkstemp(randpath)) == -1 ||
+			fchmod(fd, ~(_getumask()) & 0666) ||
 			!(fp = fdopen(fd, payload->tempfile_openmode))) {
 		unlink(randpath);
-		close(fd);
+		if(fd >= 0) {
+			close(fd);
+		}
 		_alpm_log(payload->handle, ALPM_LOG_ERROR,
 				_("failed to create temporary file for download\n"));
 		return NULL;
@@ -310,14 +323,16 @@ static FILE *create_tempfile(struct dload_payload *payload, const char *localpat
 	return fp;
 }
 
+/* RFC1123 states applications should support this length */
+#define HOSTNAME_SIZE 256
+
 static int curl_download_internal(struct dload_payload *payload,
 		const char *localpath, char **final_file)
 {
 	int ret = -1;
 	FILE *localf = NULL;
 	char *effective_url;
-	/* RFC1123 states applications should support this length */
-	char hostname[256];
+	char hostname[HOSTNAME_SIZE];
 	char error_buffer[CURL_ERROR_SIZE] = {0};
 	struct stat st;
 	long timecond, respcode = 0, remote_time = -1;
@@ -330,9 +345,10 @@ static int curl_download_internal(struct dload_payload *payload,
 
 	payload->tempfile_openmode = "wb";
 	if(!payload->remote_name) {
-		payload->remote_name = strdup(get_filename(payload->fileurl));
+		STRDUP(payload->remote_name, get_filename(payload->fileurl),
+				RET_ERR(handle, ALPM_ERR_MEMORY, -1));
 	}
-	if(!payload->remote_name || curl_gethost(payload->fileurl, hostname) != 0) {
+	if(curl_gethost(payload->fileurl, hostname, sizeof(hostname)) != 0) {
 		_alpm_log(handle, ALPM_LOG_ERROR, _("url '%s' is invalid\n"), payload->fileurl);
 		RET_ERR(handle, ALPM_ERR_SERVER_BAD_URL, -1);
 	}
@@ -388,8 +404,15 @@ static int curl_download_internal(struct dload_payload *payload,
 		case CURLE_OK:
 			/* get http/ftp response code */
 			curl_easy_getinfo(curl, CURLINFO_RESPONSE_CODE, &respcode);
+			_alpm_log(handle, ALPM_LOG_DEBUG, "response code: %ld\n", respcode);
 			if(respcode >= 400) {
 				payload->unlink_on_fail = 1;
+				/* non-translated message is same as libcurl */
+				snprintf(error_buffer, sizeof(error_buffer),
+						"The requested URL returned error: %ld", respcode);
+				_alpm_log(handle, ALPM_LOG_ERROR,
+						_("failed retrieving file '%s' from %s : %s\n"),
+						payload->remote_name, hostname, error_buffer);
 				goto cleanup;
 			}
 			break;
@@ -398,15 +421,16 @@ static int curl_download_internal(struct dload_payload *payload,
 			if(dload_interrupted == ABORT_OVER_MAXFILESIZE) {
 				payload->curlerr = CURLE_FILESIZE_EXCEEDED;
 				handle->pm_errno = ALPM_ERR_LIBCURL;
-				/* the hardcoded 'size exceeded' message is same as libcurl's normal */
+				/* use the 'size exceeded' message from libcurl */
 				_alpm_log(handle, ALPM_LOG_ERROR,
 						_("failed retrieving file '%s' from %s : %s\n"),
-						payload->remote_name, hostname, "Maximum file size exceeded");
+						payload->remote_name, hostname,
+						curl_easy_strerror(CURLE_FILESIZE_EXCEEDED));
 			}
 			goto cleanup;
 		default:
 			/* delete zero length downloads */
-			if(stat(payload->tempfile_name, &st) == 0 && st.st_size == 0) {
+			if(fstat(fileno(localf), &st) == 0 && st.st_size == 0) {
 				payload->unlink_on_fail = 1;
 			}
 			if(!payload->errors_ok) {
