@@ -66,7 +66,7 @@ int trans_init(alpm_transflag_t flags, int check_valid)
 void trans_init_error(void)
 {
 	enum _alpm_errno_t err = alpm_errno(config->handle);
-	pm_fprintf(stderr, ALPM_LOG_ERROR, _("failed to init transaction (%s)\n"),
+	pm_printf(ALPM_LOG_ERROR, _("failed to init transaction (%s)\n"),
 			alpm_strerror(err));
 	if(err == ALPM_ERR_HANDLE_LOCK) {
 		fprintf(stderr, _("  if you're sure a package manager is not already\n"
@@ -78,7 +78,7 @@ void trans_init_error(void)
 int trans_release(void)
 {
 	if(alpm_trans_release(config->handle) == -1) {
-		pm_fprintf(stderr, ALPM_LOG_ERROR, _("failed to release transaction (%s)\n"),
+		pm_printf(ALPM_LOG_ERROR, _("failed to release transaction (%s)\n"),
 				alpm_strerror(alpm_errno(config->handle)));
 		return -1;
 	}
@@ -488,34 +488,47 @@ void string_display(const char *title, const char *string)
 }
 
 static void table_print_line(const alpm_list_t *line,
-		const alpm_list_t *formats)
+		size_t colcount, size_t *widths)
 {
-	const alpm_list_t *curformat = formats;
-	const alpm_list_t *curcell = line;
+	size_t i;
+	const alpm_list_t *curcell;
 
-	while(curcell && curformat) {
-		printf(alpm_list_getdata(curformat), alpm_list_getdata(curcell));
-		curcell = alpm_list_next(curcell);
-		curformat = alpm_list_next(curformat);
+	for(i = 0, curcell = line; curcell && i < colcount;
+			i++, curcell = alpm_list_next(curcell)) {
+		const char *value = curcell->data;
+		size_t len = string_length(value);
+		/* silly printf requires padding size to be an int */
+		int padding = (int)widths[i] - (int)len;
+		if(padding < 0) {
+			padding = 0;
+		}
+		/* left-align all but the last column */
+		if(i + 1 < colcount) {
+			printf("%s%*s", value, padding, "");
+		} else {
+			printf("%*s%s", padding, "", value);
+		}
 	}
 
 	printf("\n");
 }
 
-/* creates format strings by checking max cell lengths in cols */
-static alpm_list_t *table_create_format(const alpm_list_t *header,
-		const alpm_list_t *rows)
+/* find the max string width of each column */
+static size_t table_calc_widths(const alpm_list_t *header,
+		const alpm_list_t *rows, size_t totalcols, size_t **widths)
 {
-	alpm_list_t *formats = NULL;
 	const alpm_list_t *i;
 	const unsigned short padding = 2;
-	size_t curcol, totalcols, totalwidth = 0;
+	size_t curcol, totalwidth = 0;
 	size_t *colwidths;
 
-	totalcols = alpm_list_count(header);
+	if(totalcols <= 0) {
+		return 0;
+	}
+
 	colwidths = malloc(totalcols * sizeof(size_t));
 	if(!colwidths) {
-		return NULL;
+		return 0;
 	}
 	/* header determines column count and initial values of longest_strs */
 	for(i = header, curcol = 0; i; i = alpm_list_next(i), curcol++) {
@@ -536,30 +549,16 @@ static alpm_list_t *table_create_format(const alpm_list_t *header,
 		}
 	}
 
-	/* now use the column width info to generate format strings */
-	for(curcol = 0; curcol < totalcols; curcol++) {
-		const char *display;
-		char *formatstr;
-		size_t colwidth = colwidths[curcol] + padding;
-		totalwidth += colwidth;
-
-		/* right align the last column for a cleaner table display */
-		display = (curcol + 1 < totalcols) ? "%%-%ds" : "%%%ds";
-		pm_asprintf(&formatstr, display, colwidth);
-
-		formats = alpm_list_add(formats, formatstr);
+	for(i = header, curcol = 0; i; i = alpm_list_next(i), curcol++) {
+		/* pad everything but the last column */
+		if(curcol + 1 < totalcols) {
+			colwidths[curcol] += padding;
+		}
+		totalwidth += colwidths[curcol];
 	}
 
-	free(colwidths);
-
-	/* return NULL if terminal is not wide enough */
-	if(totalwidth > getcols()) {
-		fprintf(stderr, _("insufficient columns available for table display\n"));
-		FREELIST(formats);
-		return NULL;
-	}
-
-	return formats;
+	*widths = colwidths;
+	return totalwidth;
 }
 
 /** Displays the list in table format
@@ -576,14 +575,21 @@ int table_display(const char *title, const alpm_list_t *header,
 		const alpm_list_t *rows)
 {
 	const alpm_list_t *i;
-	alpm_list_t *formats;
+	size_t *widths = NULL, totalcols, totalwidth;
 
 	if(rows == NULL || header == NULL) {
 		return 0;
 	}
 
-	formats = table_create_format(header, rows);
-	if(formats == NULL) {
+	totalcols = alpm_list_count(header);
+	totalwidth = table_calc_widths(header, rows, totalcols, &widths);
+	/* return -1 if terminal is not wide enough */
+	if(totalwidth > getcols()) {
+		pm_printf(ALPM_LOG_WARNING,
+				_("insufficient columns available for table display\n"));
+		return -1;
+	}
+	if(!totalwidth || !widths) {
 		return -1;
 	}
 
@@ -591,14 +597,14 @@ int table_display(const char *title, const alpm_list_t *header,
 		printf("%s\n\n", title);
 	}
 
-	table_print_line(header, formats);
+	table_print_line(header, totalcols, widths);
 	printf("\n");
 
 	for(i = rows; i; i = alpm_list_next(i)) {
-		table_print_line(alpm_list_getdata(i), formats);
+		table_print_line(alpm_list_getdata(i), totalcols, widths);
 	}
 
-	FREELIST(formats);
+	free(widths);
 	return 0;
 }
 
@@ -737,7 +743,7 @@ void signature_display(const char *title, alpm_siglist_t *siglist)
 			ret = pm_asprintf(&sigline, _("%s, %s from \"%s\""),
 					status, validity, name);
 			if(ret == -1) {
-				pm_fprintf(stderr, ALPM_LOG_ERROR,  _("failed to allocate string\n"));
+				pm_printf(ALPM_LOG_ERROR,  _("failed to allocate string\n"));
 				continue;
 			}
 			indentprint(sigline, len);
@@ -1038,19 +1044,19 @@ void print_packages(const alpm_list_t *packages)
 		char *string = strdup(config->print_format);
 		char *temp = string;
 		/* %n : pkgname */
-		if(strstr(temp,"%n")) {
+		if(strstr(temp, "%n")) {
 			string = strreplace(temp, "%n", alpm_pkg_get_name(pkg));
 			free(temp);
 			temp = string;
 		}
 		/* %v : pkgver */
-		if(strstr(temp,"%v")) {
+		if(strstr(temp, "%v")) {
 			string = strreplace(temp, "%v", alpm_pkg_get_version(pkg));
 			free(temp);
 			temp = string;
 		}
 		/* %l : location */
-		if(strstr(temp,"%l")) {
+		if(strstr(temp, "%l")) {
 			char *pkgloc = pkg_get_location(pkg);
 			string = strreplace(temp, "%l", pkgloc);
 			free(pkgloc);
@@ -1058,7 +1064,7 @@ void print_packages(const alpm_list_t *packages)
 			temp = string;
 		}
 		/* %r : repo */
-		if(strstr(temp,"%r")) {
+		if(strstr(temp, "%r")) {
 			const char *repo = "local";
 			alpm_db_t *db = alpm_pkg_get_db(pkg);
 			if(db) {
@@ -1069,7 +1075,7 @@ void print_packages(const alpm_list_t *packages)
 			temp = string;
 		}
 		/* %s : size */
-		if(strstr(temp,"%s")) {
+		if(strstr(temp, "%s")) {
 			char *size;
 			pm_asprintf(&size, "%jd", (intmax_t)pkg_get_size(pkg));
 			string = strreplace(temp, "%s", size);
@@ -1152,14 +1158,15 @@ static int parseindex(char *s, int *val, int min, int max)
 	int n = strtol(s, &endptr, 10);
 	if(*endptr == '\0') {
 		if(n < min || n > max) {
-			fprintf(stderr, _("Invalid value: %d is not between %d and %d\n"),
+			pm_printf(ALPM_LOG_ERROR,
+					_("invalid value: %d is not between %d and %d\n"),
 					n, min, max);
 			return -1;
 		}
 		*val = n;
 		return 0;
 	} else {
-		fprintf(stderr, _("Invalid number: %s\n"), s);
+		pm_printf(ALPM_LOG_ERROR, _("invalid number: %s\n"), s);
 		return -1;
 	}
 }
@@ -1411,20 +1418,7 @@ int pm_printf(alpm_loglevel_t level, const char *format, ...)
 
 	/* print the message using va_arg list */
 	va_start(args, format);
-	ret = pm_vfprintf(stdout, level, format, args);
-	va_end(args);
-
-	return ret;
-}
-
-int pm_fprintf(FILE *stream, alpm_loglevel_t level, const char *format, ...)
-{
-	int ret;
-	va_list args;
-
-	/* print the message using va_arg list */
-	va_start(args, format);
-	ret = pm_vfprintf(stream, level, format, args);
+	ret = pm_vfprintf(stderr, level, format, args);
 	va_end(args);
 
 	return ret;
@@ -1438,7 +1432,7 @@ int pm_asprintf(char **string, const char *format, ...)
 	/* print the message using va_arg list */
 	va_start(args, format);
 	if(vasprintf(string, format, args) == -1) {
-		pm_fprintf(stderr, ALPM_LOG_ERROR,  _("failed to allocate string\n"));
+		pm_printf(ALPM_LOG_ERROR,  _("failed to allocate string\n"));
 		ret = -1;
 	}
 	va_end(args);
