@@ -19,6 +19,7 @@
 
 #include "config.h"
 
+#include <ctype.h>
 #include <getopt.h>
 #include <stdio.h>
 #include <string.h>
@@ -26,63 +27,171 @@
 #include <alpm.h>
 #include <alpm_list.h>
 
-/* output */
-char *provides     = " provides";
-char *unresolvable = " [unresolvable]";
-char *branch_tip1  = "|--";
-char *branch_tip2  = "+--";
-int   indent_size  = 3;
+#define LINE_MAX     512
 
-/* color */
-char *branch1_color = "\033[0;33m"; /* yellow */
-char *branch2_color = "\033[0;37m"; /* white */
-char *leaf1_color   = "\033[1;32m"; /* bold green */
-char *leaf2_color   = "\033[0;32m"; /* green */
-char *color_off     = "\033[0m";
+/* output */
+struct graph_style {
+	const char *provides;
+	const char *tip1;
+	const char *tip2;
+	int indent;
+};
+
+static struct graph_style graph_default = {
+	" provides",
+	"|--",
+	"+--",
+	3
+};
+
+static struct graph_style graph_linear = {
+	"",
+	"",
+	"",
+	0
+};
+
+/* color choices */
+struct color_choices {
+	const char *branch1;
+	const char *branch2;
+	const char *leaf1;
+	const char *leaf2;
+	const char *off;
+};
+
+static struct color_choices use_color = {
+	"\033[0;33m", /* yellow */
+	"\033[0;37m", /* white */
+	"\033[1;32m", /* bold green */
+	"\033[0;32m", /* green */
+	"\033[0m"
+};
+
+static struct color_choices no_color = {
+	"",
+	"",
+	"",
+	"",
+	""
+};
+
+/* long operations */
+enum {
+	OP_CONFIG = 1000
+};
 
 /* globals */
-pmdb_t *db_local;
+alpm_handle_t *handle = NULL;
 alpm_list_t *walked = NULL;
 alpm_list_t *provisions = NULL;
 
 /* options */
-int color = 0;
+struct color_choices *color = &no_color;
+struct graph_style *style = &graph_default;
 int graphviz = 0;
-int linear = 0;
 int max_depth = -1;
 int reverse = 0;
 int unique = 0;
-char *dbpath = NULL;
+int searchsyncs = 0;
+const char *dbpath = DBPATH;
+const char *configfile = CONFFILE;
 
-static int alpm_local_init(void)
+#ifndef HAVE_STRNDUP
+/* A quick and dirty implementation derived from glibc */
+static size_t strnlen(const char *s, size_t max)
 {
-	int ret;
+    register const char *p;
+    for(p = s; *p && max--; ++p);
+    return (p - s);
+}
 
-	ret = alpm_initialize();
-	if(ret != 0) {
-		return(ret);
+char *strndup(const char *s, size_t n)
+{
+  size_t len = strnlen(s, n);
+  char *new = (char *) malloc(len + 1);
+
+  if(new == NULL)
+    return NULL;
+
+  new[len] = '\0';
+  return (char *)memcpy(new, s, len);
+}
+#endif
+
+static char *strtrim(char *str)
+{
+	char *pch = str;
+
+	if(str == NULL || *str == '\0') {
+		/* string is empty, so we're done. */
+		return str;
 	}
 
-	ret = alpm_option_set_root(ROOTDIR);
-	if(ret != 0) {
-		return(ret);
+	while(isspace((unsigned char)*pch)) {
+		pch++;
+	}
+	if(pch != str) {
+		size_t len = strlen(pch);
+		if(len) {
+			memmove(str, pch, len + 1);
+		} else {
+			*str = '\0';
+		}
 	}
 
-	if(dbpath) {
-		ret = alpm_option_set_dbpath(dbpath);
-	} else {
-		ret = alpm_option_set_dbpath(DBPATH);
-	}
-	if(ret != 0) {
-		return(ret);
+	/* check if there wasn't anything but whitespace in the string. */
+	if(*str == '\0') {
+		return str;
 	}
 
-	db_local = alpm_option_get_localdb();
-	if(!db_local) {
-		return(1);
+	pch = (str + (strlen(str) - 1));
+	while(isspace((unsigned char)*pch)) {
+		pch--;
+	}
+	*++pch = '\0';
+
+	return str;
+}
+
+static int register_syncs(void) {
+	FILE *fp;
+	char *ptr, *section = NULL;
+	char line[LINE_MAX];
+	const alpm_siglevel_t level = ALPM_SIG_DATABASE | ALPM_SIG_DATABASE_OPTIONAL;
+
+	fp = fopen(configfile, "r");
+	if(!fp) {
+		fprintf(stderr, "error: config file %s could not be read\n", configfile);
+		return 1;
 	}
 
-	return(0);
+	while(fgets(line, LINE_MAX, fp)) {
+		strtrim(line);
+
+		if(line[0] == '#' || !strlen(line)) {
+			continue;
+		}
+
+		if((ptr = strchr(line, '#'))) {
+			*ptr = '\0';
+			strtrim(line);
+		}
+
+		if(line[0] == '[' && line[strlen(line) - 1] == ']') {
+			free(section);
+			section = strndup(&line[1], strlen(line) - 2);
+
+			if(section && strcmp(section, "options") != 0) {
+				alpm_db_register_sync(handle, section, level);
+			}
+		}
+	}
+
+	free(section);
+	fclose(fp);
+
+	return 0;
 }
 
 static int parse_options(int argc, char *argv[])
@@ -90,7 +199,7 @@ static int parse_options(int argc, char *argv[])
 	int opt, option_index = 0;
 	char *endptr = NULL;
 
-	static struct option opts[] = {
+	static const struct option opts[] = {
 		{"dbpath",  required_argument,    0, 'b'},
 		{"color",   no_argument,          0, 'c'},
 		{"depth",   required_argument,    0, 'd'},
@@ -98,21 +207,27 @@ static int parse_options(int argc, char *argv[])
 		{"help",    no_argument,          0, 'h'},
 		{"linear",  no_argument,          0, 'l'},
 		{"reverse", no_argument,          0, 'r'},
+		{"sync",    no_argument,          0, 'S'},
 		{"unique",  no_argument,          0, 'u'},
+
+		{"config",  required_argument,    0, OP_CONFIG},
 		{0, 0, 0, 0}
 	};
 
-	while((opt = getopt_long(argc, argv, "b:cd:ghlru", opts, &option_index))) {
+	while((opt = getopt_long(argc, argv, "b:cd:ghlrsu", opts, &option_index))) {
 		if(opt < 0) {
 			break;
 		}
 
 		switch(opt) {
+			case OP_CONFIG:
+				configfile = optarg;
+				break;
 			case 'b':
-				dbpath = strdup(optarg);
+				dbpath = optarg;
 				break;
 			case 'c':
-				color = 1;
+				color = &use_color;
 				break;
 			case 'd':
 				/* validate depth */
@@ -126,40 +241,30 @@ static int parse_options(int argc, char *argv[])
 				graphviz = 1;
 				break;
 			case 'l':
-				linear = 1;
+				style = &graph_linear;
 				break;
 			case 'r':
 				reverse = 1;
 				break;
+			case 's':
+				searchsyncs = 1;
+				break;
 			case 'u':
-				unique = linear = 1;
+				unique = 1;
+				style = &graph_linear;
 				break;
 			case 'h':
 			case '?':
 			default:
-				return(1);
+				return 1;
 		}
 	}
 
 	if(!argv[optind]) {
-		return(1);
+		return 1;
 	}
 
-	if(!color) {
-		branch1_color = "";
-		branch2_color = "";
-		leaf1_color   = "";
-		leaf2_color   = "";
-		color_off     = "";
-	}
-	if(linear) {
-		provides     = "";
-		branch_tip1  = "";
-		branch_tip2  = "";
-		indent_size  = 0;
-	}
-
-	return(0);
+	return 0;
 }
 
 static void usage(void)
@@ -170,27 +275,25 @@ static void usage(void)
 			"  -c, --color          colorize output\n"
 			"  -d, --depth <#>      limit the depth of recursion\n"
 			"  -g, --graph          generate output for graphviz's dot\n"
+			"  -h, --help           display this help message\n"
 			"  -l, --linear         enable linear output\n"
 			"  -r, --reverse        show reverse dependencies\n"
-			"  -u, --unique         show dependencies with no duplicates (implies -l)\n\n"
-			"  -h, --help           display this help message\n");
+			"  -s, --sync           search sync DBs instead of local\n"
+			"  -u, --unique         show dependencies with no duplicates (implies -l)\n"
+			"      --config <path>  set an alternate configuration file\n");
 }
 
 static void cleanup(void)
 {
-	if(dbpath) {
-		free(dbpath);
-	}
-
 	alpm_list_free(walked);
 	alpm_list_free(provisions);
-	alpm_release();
+	alpm_release(handle);
 }
 
 /* pkg provides provision */
 static void print_text(const char *pkg, const char *provision, int depth)
 {
-	int indent_sz = (depth + 1) * indent_size;
+	int indent_sz = (depth + 1) * style->indent;
 
 	if(!pkg && !provision) {
 		/* not much we can do */
@@ -199,17 +302,17 @@ static void print_text(const char *pkg, const char *provision, int depth)
 
 	if(!pkg && provision) {
 		/* we failed to resolve provision */
-		printf("%s%*s%s%s%s%s%s\n", branch1_color, indent_sz, branch_tip1,
-				leaf1_color, provision, branch1_color, unresolvable, color_off);
+		printf("%s%*s%s%s%s [unresolvable]%s\n", color->branch1, indent_sz,
+				style->tip1, color->leaf1, provision, color->branch1, color->off);
 	} else if(provision && strcmp(pkg, provision) != 0) {
 		/* pkg provides provision */
-		printf("%s%*s%s%s%s%s %s%s%s\n", branch2_color, indent_sz, branch_tip2,
-				leaf1_color, pkg, leaf2_color, provides, leaf1_color, provision,
-				color_off);
+		printf("%s%*s%s%s%s%s %s%s%s\n", color->branch2, indent_sz, style->tip2,
+				color->leaf1, pkg, color->leaf2, style->provides, color->leaf1, provision,
+				color->off);
 	} else {
 		/* pkg is a normal package */
-		printf("%s%*s%s%s%s\n", branch1_color, indent_sz, branch_tip1, leaf1_color,
-				pkg, color_off);
+		printf("%s%*s%s%s%s\n", color->branch1, indent_sz, style->tip1, color->leaf1,
+				pkg, color->off);
 	}
 }
 
@@ -255,19 +358,31 @@ static void print_end(void)
 	}
 }
 
+static alpm_pkg_t *get_pkg_from_dbs(alpm_list_t *dbs, const char *needle) {
+	alpm_list_t *i;
+	alpm_pkg_t *ret;
+
+	for(i = dbs; i; i = alpm_list_next(i)) {
+		ret = alpm_db_get_pkg(alpm_list_getdata(i), needle);
+		if(ret) {
+			return ret;
+		}
+	}
+	return NULL;
+}
 
 /**
  * walk dependencies in reverse, showing packages which require the target
  */
-static void walk_reverse_deps(pmpkg_t *pkg, int depth)
+static void walk_reverse_deps(alpm_list_t *dblist, alpm_pkg_t *pkg, int depth)
 {
 	alpm_list_t *required_by, *i;
 
-	if((max_depth >= 0) && (depth == max_depth + 1)) {
+	if(!pkg || ((max_depth >= 0) && (depth == max_depth + 1))) {
 		return;
 	}
 
-	walked = alpm_list_add(walked, (void*)alpm_pkg_get_name(pkg));
+	walked = alpm_list_add(walked, (void *)alpm_pkg_get_name(pkg));
 	required_by = alpm_pkg_compute_requiredby(pkg);
 
 	for(i = required_by; i; i = alpm_list_next(i)) {
@@ -281,7 +396,7 @@ static void walk_reverse_deps(pmpkg_t *pkg, int depth)
 			}
 		} else {
 			print(alpm_pkg_get_name(pkg), pkgname, NULL, depth);
-			walk_reverse_deps(alpm_db_get_pkg(db_local, pkgname), depth + 1);
+			walk_reverse_deps(dblist, get_pkg_from_dbs(dblist, pkgname), depth + 1);
 		}
 	}
 
@@ -291,7 +406,7 @@ static void walk_reverse_deps(pmpkg_t *pkg, int depth)
 /**
  * walk dependencies, showing dependencies of the target
  */
-static void walk_deps(pmpkg_t *pkg, int depth)
+static void walk_deps(alpm_list_t *dblist, alpm_pkg_t *pkg, int depth)
 {
 	alpm_list_t *i;
 
@@ -299,12 +414,11 @@ static void walk_deps(pmpkg_t *pkg, int depth)
 		return;
 	}
 
-	walked = alpm_list_add(walked, (void*)alpm_pkg_get_name(pkg));
+	walked = alpm_list_add(walked, (void *)alpm_pkg_get_name(pkg));
 
 	for(i = alpm_pkg_get_depends(pkg); i; i = alpm_list_next(i)) {
-		pmdepend_t *depend = alpm_list_getdata(i);
-		pmpkg_t *provider = alpm_find_satisfier(alpm_db_get_pkgcache(db_local),
-				alpm_dep_get_name(depend));
+		alpm_depend_t *depend = alpm_list_getdata(i);
+		alpm_pkg_t *provider = alpm_find_dbs_satisfier(handle, dblist, depend->name);
 
 		if(provider) {
 			const char *provname = alpm_pkg_get_name(provider);
@@ -313,41 +427,56 @@ static void walk_deps(pmpkg_t *pkg, int depth)
 				/* if we've already seen this package, don't print in "unique" output
 				 * and don't recurse */
 				if(!unique) {
-					print(alpm_pkg_get_name(pkg), provname, alpm_dep_get_name(depend), depth);
+					print(alpm_pkg_get_name(pkg), provname, depend->name, depth);
 				}
 			} else {
-				print(alpm_pkg_get_name(pkg), provname, alpm_dep_get_name(depend), depth);
-				walk_deps(provider, depth + 1);
+				print(alpm_pkg_get_name(pkg), provname, depend->name, depth);
+				walk_deps(dblist, provider, depth + 1);
 			}
 		} else {
 			/* unresolvable package */
-			print(alpm_pkg_get_name(pkg), NULL, alpm_dep_get_name(depend), depth);
+			print(alpm_pkg_get_name(pkg), NULL, depend->name, depth);
 		}
 	}
 }
 
 int main(int argc, char *argv[])
 {
-	int ret;
+	int freelist = 0, ret = 0;
+	enum _alpm_errno_t err;
 	const char *target_name;
-	pmpkg_t *pkg;
+	alpm_pkg_t *pkg;
+	alpm_list_t *dblist = NULL;
 
-	ret = parse_options(argc, argv);
-	if(ret != 0) {
+	if(parse_options(argc, argv) != 0) {
 		usage();
+		ret = 1;
 		goto finish;
 	}
 
-	ret = alpm_local_init();
-	if(ret != 0) {
-		fprintf(stderr, "error: cannot initialize alpm: %s\n", alpm_strerrorlast());
+	handle = alpm_initialize(ROOTDIR, dbpath, &err);
+	if(!handle) {
+		fprintf(stderr, "error: cannot initialize alpm: %s\n",
+				alpm_strerror(err));
+		ret = 1;
 		goto finish;
 	}
 
+	if(searchsyncs) {
+		if(register_syncs() != 0) {
+			ret = 1;
+			goto finish;
+		}
+		dblist = alpm_option_get_syncdbs(handle);
+	} else {
+		dblist = alpm_list_add(dblist, alpm_option_get_localdb(handle));
+		freelist = 1;
+	}
+
 	/* we only care about the first non option arg for walking */
 	target_name = argv[optind];
 
-	pkg = alpm_find_satisfier(alpm_db_get_pkgcache(db_local), target_name);
+	pkg = alpm_find_dbs_satisfier(handle, dblist, target_name);
 	if(!pkg) {
 		fprintf(stderr, "error: package '%s' not found\n", target_name);
 		ret = 1;
@@ -357,16 +486,20 @@ int main(int argc, char *argv[])
 	print_start(alpm_pkg_get_name(pkg), target_name);
 
 	if(reverse) {
-		walk_reverse_deps(pkg, 1);
+		walk_reverse_deps(dblist, pkg, 1);
 	} else {
-		walk_deps(pkg, 1);
+		walk_deps(dblist, pkg, 1);
 	}
 
 	print_end();
 
+	if(freelist) {
+		alpm_list_free(dblist);
+	}
+
 finish:
 	cleanup();
-	return(ret);
+	return ret;
 }
 
 /* vim: set ts=2 sw=2 noet: */
